import requests


class EventGateway:
    def __init__(self, url="http://localhost", space="default", adminPort=4001,
                 clientPort=4000):
        self.url = url
        self.space = space
        self.adminPort = adminPort
        self.clientPort = clientPort

    # UTILS #

    def checkConnection(self):
        egHealthCheckUrl = "{}:{}/v1/status".format(
            self.url, str(self.adminPort))
        try:
            r = requests.get(egHealthCheckUrl)
            if r.status_code == 200:
                return True
        except requests.exceptions.ConnectionError:
            print("Cannot connect to {}".format(egHealthCheckUrl))
            return False

    def getAdminUrl(self):
        return "{}:{}/v1/spaces/{}".format(
            self.url, str(self.adminPort), self.space)

    def getClientUrl(self):
        return "{}:{}".format(self.url, str(self.clientPort))

    def printConfig(self):
        return """
            Event-gateway information:
            URL: {}
            Admin port: {}
            Client port: {}
            Space: {}""".format(
                self.url, self.adminPort, self.clientPort, self.space)

    # CREATE STUFF #

    def createEventType(self, params):
        return self.makePostRequest("eventtypes", params, rsrcId="name")

    def createFunction(self, params):
        return self.makePostRequest("functions", params)

    def createSubscription(self, params):
        return self.makePostRequest("subscriptions", params)

    def createCORS(self, params):
        return self.makePostRequest("cors", params, rsrcId="path")

    def createConnection(self, params):
        return self.makePostRequest("connections", params, rsrcId="type")

    # GENERIC POST REQUEST #

    def makePostRequest(self, resourceType, params, rsrcId="functionId"):
        url = self.getAdminUrl() + "/" + resourceType
        r = requests.post(url, json=params)
        if r.status_code == 201:
            print("{} {} created successfully".format(
                resourceType.title(), params[rsrcId]))
            return r.json()
        elif r.status_code == 409:
            print("{} {} already exists".format(
                resourceType.title(), params[rsrcId]))
            return False
        else:
            print("Issue while creating {} {}\n".format(
                resourceType.title(), params[rsrcId]))
            print(r.json())
            return False

    # EMIT #

    def emit(self, cloudEvent, path="/",
             headers={"Content-type": "application/json"}):
        url = self.getClientUrl() + path
        r = requests.post(url, json=cloudEvent, headers=headers)
        return r



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="event-gateway-sdk",
    version="1.0.0",
    author="Johann BICH",
    author_email="johannbich@gmail.com",
    description="Event Gateway Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kalote/event-gateway-sdk",
    packages=find_packages(exclude=('tests')),
    install_requires=["requests"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import os
import json
import unittest
import responses
from .context import eventgateway

with open(os.path.dirname(__file__)+'/testData.json', 'r') as f:
    testData = json.load(f)


class EgHappyPath(unittest.TestCase):
    def setUp(self):
        self.eg = eventgateway.EventGateway()
        self.egUrl = "http://localhost:4001"
        self.test_checkConnection()

    @responses.activate
    def test_checkConnection(self):
        responses.add(responses.GET, "{}/v1/status".format(self.egUrl),
                      status=200)
        self.assertTrue(self.eg.checkConnection())

    @responses.activate
    def test_createEventType(self):
        responses.add(responses.POST,
                      "{}/v1/spaces/default/eventtypes".format(self.egUrl),
                      json=testData["eventtypeValid"], status=201)
        self.assertEqual(
            self.eg.createEventType(testData["eventtype"]),
            testData["eventtypeValid"]
        )

    @responses.activate
    def test_createFunction(self):
        responses.add(responses.POST,
                      "{}/v1/spaces/default/functions".format(self.egUrl),
                      json=testData["functionValid"], status=201)
        self.assertEqual(
            self.eg.createFunction(testData["function"]),
            testData["functionValid"]
        )

    @responses.activate
    def test_createSubscription(self):
        responses.add(responses.POST,
                      "{}/v1/spaces/default/subscriptions".format(self.egUrl),
                      json=testData["subscriptionValid"], status=201)
        self.assertEqual(
            self.eg.createSubscription(testData["subscription"]),
            testData["subscriptionValid"]
        )
